# Copyright 2024 Deepgram SDK contributors. All Rights Reserved.
# Use of this source code is governed by a MIT license that can be found in the LICENSE file.
# SPDX-License-Identifier: MIT

from .client import SpeakClient
from .client import AsyncSpeakClient
from .client import SpeakOptions
from .client import SpeakResponse
from .client import (
    SpeakSource,
    TextSource,
    SpeakStreamSource,
)

from ...options import DeepgramClientOptions
