# Copyright 2024 Deepgram SDK contributors. All Rights Reserved.
# Use of this source code is governed by a MIT license that can be found in the LICENSE file.
# SPDX-License-Identifier: MIT

from .client import SpeakClient
from .async_client import AsyncSpeakClient
from .options import SpeakOptions, SpeakSource, TextSource, SpeakStreamSource
from .response import SpeakResponse
from ....options import DeepgramClientOptions
